﻿using Microsoft.IoT.Lightning.Providers;
using Windows.Devices;
using Windows.Devices.Gpio;
using System;

namespace TNKLabo
{
	public enum AM50288H_Mark
	{
		Asterisk = 57,
		Diamond = 58,
		Square = 59,
		Anntena = 60,
		Battery = 61,
		Next = 136,
		Bell = 137,
		Cycle = 138,
		Note = 140,
		Speaker = 141,
	}

	[Flags]
	public enum SegmentPosition
	{
		None = 0,
		Top = 1,
		RightTop = 2,
		RightBottom = 4,
		Bottom = 8,
		LeftBottom = 16,
		LeftTop = 32,
		MiddleLeft = 64,
		MiddleRight = 128,
		TopLeft = 256,
		TopCenter = 512,
		TopRight = 1024,
		BottomRight = 2048,
		BottomCenter = 4096,
		BottomLeft = 8192,

		A = Top,
		B = RightTop,
		C = RightBottom,
		D = Bottom,
		E = LeftBottom,
		F = LeftTop,
		G1 = MiddleLeft,
		G2 = MiddleRight,
		J = TopLeft,
		K = TopCenter,
		L = TopRight,
		M = BottomRight,
		N = BottomCenter,
		P = BottomLeft,
	}

	public sealed class AM50288H
	{
		private const int Digits = 8;
		private const int Segments = 14;

		private enum SegmentBit
		{
			SGB_1J = 0, SGB_1G2 = 1, SGB_1P = 2, SGB_1N = 3, SGB_1M = 4,
			SGB_1L = 5, SGB_1K = 6, SGB_2J = 7, SGB_2G2 = 8, SGB_2P = 9,
			SGB_2N = 10, SGB_2M = 11, SGB_2L = 12, SGB_2K = 13, SGB_3J = 14,
			SGB_3G2 = 15, SGB_3P = 16, SGB_3N = 17, SGB_3M = 18, SGB_3L = 19,
			SGB_3K = 20, SGB_4J = 21, SGB_4G2 = 22, SGB_4P = 23, SGB_4N = 24,
			SGB_4M = 25, SGB_4L = 26, SGB_4K = 27, SGB_5J = 28, SGB_5G2 = 29,
			SGB_5P = 30, SGB_5N = 31, SGB_5M = 32, SGB_5L = 33, SGB_5K = 34,
			SGB_6J = 35, SGB_6G2 = 36, SGB_6P = 37, SGB_6N = 38, SGB_6M = 39,
			SGB_6L = 40, SGB_6K = 41, SGB_7J = 42, SGB_7G2 = 43, SGB_7P = 44,
			SGB_7N = 45, SGB_7M = 46, SGB_7L = 47, SGB_7K = 48, SGB_8J = 49,
			SGB_8G2 = 50, SGB_8P = 51, SGB_8N = 52, SGB_8M = 53, SGB_8L = 54,
			SGB_8K = 55, SGB_1F = 80, SGB_1G1 = 81, SGB_1E = 82, SGB_1D = 83,
			SGB_1C = 84, SGB_1B = 85, SGB_1A = 86, SGB_2F = 87, SGB_2G1 = 88,
			SGB_2E = 89, SGB_2D = 90, SGB_2C = 91, SGB_2B = 92, SGB_2A = 93,
			SGB_3F = 94, SGB_3G1 = 95, SGB_3E = 96, SGB_3D = 97, SGB_3C = 98,
			SGB_3B = 99, SGB_3A = 100, SGB_4F = 101, SGB_4G1 = 102, SGB_4E = 103,
			SGB_4D = 104, SGB_4C = 105, SGB_4B = 106, SGB_4A = 107, SGB_5F = 108,
			SGB_5G1 = 109, SGB_5E = 110, SGB_5D = 111, SGB_5C = 112, SGB_5B = 113,
			SGB_5A = 114, SGB_6F = 115, SGB_6G1 = 116, SGB_6E = 117, SGB_6D = 118,
			SGB_6C = 119, SGB_6B = 120, SGB_6A = 121, SGB_7F = 122, SGB_7G1 = 123,
			SGB_7E = 124, SGB_7D = 125, SGB_7C = 126, SGB_7B = 127, SGB_7A = 128,
			SGB_8F = 129, SGB_8G1 = 130, SGB_8E = 131, SGB_8D = 132, SGB_8C = 133,
			SGB_8B = 134, SGB_8A = 135,
		}

		private SegmentBit[,] segarray = new SegmentBit[Digits, Segments] {
			{SegmentBit.SGB_1A, SegmentBit.SGB_1B, SegmentBit.SGB_1C, SegmentBit.SGB_1D, SegmentBit.SGB_1E, SegmentBit.SGB_1F, SegmentBit.SGB_1G1, SegmentBit.SGB_1G2, SegmentBit.SGB_1J, SegmentBit.SGB_1K, SegmentBit.SGB_1L, SegmentBit.SGB_1M, SegmentBit.SGB_1N, SegmentBit.SGB_1P},
			{SegmentBit.SGB_2A, SegmentBit.SGB_2B, SegmentBit.SGB_2C, SegmentBit.SGB_2D, SegmentBit.SGB_2E, SegmentBit.SGB_2F, SegmentBit.SGB_2G1, SegmentBit.SGB_2G2, SegmentBit.SGB_2J, SegmentBit.SGB_2K, SegmentBit.SGB_2L, SegmentBit.SGB_2M, SegmentBit.SGB_2N, SegmentBit.SGB_2P},
			{SegmentBit.SGB_3A, SegmentBit.SGB_3B, SegmentBit.SGB_3C, SegmentBit.SGB_3D, SegmentBit.SGB_3E, SegmentBit.SGB_3F, SegmentBit.SGB_3G1, SegmentBit.SGB_3G2, SegmentBit.SGB_3J, SegmentBit.SGB_3K, SegmentBit.SGB_3L, SegmentBit.SGB_3M, SegmentBit.SGB_3N, SegmentBit.SGB_3P},
			{SegmentBit.SGB_4A, SegmentBit.SGB_4B, SegmentBit.SGB_4C, SegmentBit.SGB_4D, SegmentBit.SGB_4E, SegmentBit.SGB_4F, SegmentBit.SGB_4G1, SegmentBit.SGB_4G2, SegmentBit.SGB_4J, SegmentBit.SGB_4K, SegmentBit.SGB_4L, SegmentBit.SGB_4M, SegmentBit.SGB_4N, SegmentBit.SGB_4P},
			{SegmentBit.SGB_5A, SegmentBit.SGB_5B, SegmentBit.SGB_5C, SegmentBit.SGB_5D, SegmentBit.SGB_5E, SegmentBit.SGB_5F, SegmentBit.SGB_5G1, SegmentBit.SGB_5G2, SegmentBit.SGB_5J, SegmentBit.SGB_5K, SegmentBit.SGB_5L, SegmentBit.SGB_5M, SegmentBit.SGB_5N, SegmentBit.SGB_5P},
			{SegmentBit.SGB_6A, SegmentBit.SGB_6B, SegmentBit.SGB_6C, SegmentBit.SGB_6D, SegmentBit.SGB_6E, SegmentBit.SGB_6F, SegmentBit.SGB_6G1, SegmentBit.SGB_6G2, SegmentBit.SGB_6J, SegmentBit.SGB_6K, SegmentBit.SGB_6L, SegmentBit.SGB_6M, SegmentBit.SGB_6N, SegmentBit.SGB_6P},
			{SegmentBit.SGB_7A, SegmentBit.SGB_7B, SegmentBit.SGB_7C, SegmentBit.SGB_7D, SegmentBit.SGB_7E, SegmentBit.SGB_7F, SegmentBit.SGB_7G1, SegmentBit.SGB_7G2, SegmentBit.SGB_7J, SegmentBit.SGB_7K, SegmentBit.SGB_7L, SegmentBit.SGB_7M, SegmentBit.SGB_7N, SegmentBit.SGB_7P},
			{SegmentBit.SGB_8A, SegmentBit.SGB_8B, SegmentBit.SGB_8C, SegmentBit.SGB_8D, SegmentBit.SGB_8E, SegmentBit.SGB_8F, SegmentBit.SGB_8G1, SegmentBit.SGB_8G2, SegmentBit.SGB_8J, SegmentBit.SGB_8K, SegmentBit.SGB_8L, SegmentBit.SGB_8M, SegmentBit.SGB_8N, SegmentBit.SGB_8P}
		};


		public byte PinLE { get; set; }
		public byte PinDIN { get; set; }
		public byte PinSCK { get; set; }

		private GpioPin ple, pdin, psck = null;
		private GpioPinValue[] dins = new GpioPinValue[142];

		public AM50288H()
		{
			PinLE = 17;
			PinDIN = 18;
			PinSCK = 27;
		}

		public void Start()
		{
			if (LightningProvider.IsLightningEnabled) {
				LowLevelDevicesController.DefaultProvider = LightningProvider.GetAggregateProvider();
			}

			GpioController gpc = GpioController.GetDefault();

			ple = gpc.OpenPin(PinLE);
			ple.SetDriveMode(GpioPinDriveMode.Output);

			pdin = gpc.OpenPin(PinDIN);
			pdin.SetDriveMode(GpioPinDriveMode.Output);

			psck = gpc.OpenPin(PinSCK);
			psck.SetDriveMode(GpioPinDriveMode.Output);

			for (int i = 0; i < 8; i++) SetSegment(i, SegmentPosition.None);
			Update();
		}

		public void Update()
		{
			if (psck == null) return;

			for(int i = dins.Length - 1; i >= 0; i--) {
				psck.Write(GpioPinValue.Low);
				pdin.Write(dins[i]);
				psck.Write(GpioPinValue.High);
			}

			ple.Write(GpioPinValue.High);
			ple.Write(GpioPinValue.Low);
		}

		public void Clear()
		{
			Array.Clear(dins, 0, dins.Length);
			Update();
		}

		public void SetSegmentAsBits(int pos, ushort segbits)
		{
			for(int i = 0; i < Segments; i++) {
				dins[(int)segarray[pos, i]] = (((segbits >> i) & 0x1) != 0) ? GpioPinValue.High : GpioPinValue.Low;
			}
		}

		public void SetSegment(int pos, SegmentPosition spos)
		{
			SetSegmentAsBits(pos, (ushort)spos);
		}

		public ushort GetAsciiSegment(char ascii)
		{
			ushort sb;
			switch (ascii) {
			case '0': sb = 0x243F; break;
			case '1': sb = 0x0006; break;
			case '2': sb = 0x00DB; break;
			case '3': sb = 0x008F; break;
			case '4': sb = 0x00E6; break;
			case '5': sb = 0x0869; break;
			case '6': sb = 0x00FD; break;
			case '7': sb = 0x0007; break;
			case '8': sb = 0x00FF; break;
			case '9': sb = 0x00EF; break;
			case 'A': sb = 0x00F7; break;
			case 'B': sb = 0x128F; break;
			case 'C': sb = 0x0039; break;
			case 'D': sb = 0x120F; break;
			case 'E': sb = 0x00F9; break;
			case 'F': sb = 0x0071; break;
			case 'G': sb = 0x00BD; break;
			case 'H': sb = 0x00F6; break;
			case 'I': sb = 0x1200; break;
			case 'J': sb = 0x001E; break;
			case 'K': sb = 0x0C70; break;
			case 'L': sb = 0x0038; break;
			case 'M': sb = 0x0536; break;
			case 'N': sb = 0x0936; break;
			case 'O': sb = 0x003F; break;
			case 'P': sb = 0x00F3; break;
			case 'Q': sb = 0x083F; break;
			case 'R': sb = 0x08F3; break;
			case 'S': sb = 0x00ED; break;
			case 'T': sb = 0x1201; break;
			case 'U': sb = 0x003E; break;
			case 'V': sb = 0x2430; break;
			case 'W': sb = 0x2836; break;
			case 'X': sb = 0x2D00; break;
			case 'Y': sb = 0x1500; break;
			case 'Z': sb = 0x2409; break;
			case 'a': sb = 0x1058; break;
			case 'b': sb = 0x0878; break;
			case 'c': sb = 0x00D8; break;
			case 'd': sb = 0x208E; break;
			case 'e': sb = 0x2058; break;
			case 'f': sb = 0x0071; break;
			case 'g': sb = 0x048E; break;
			case 'h': sb = 0x1070; break;
			case 'i': sb = 0x1000; break;
			case 'j': sb = 0x000E; break;
			case 'k': sb = 0x1E00; break;
			case 'l': sb = 0x0030; break;
			case 'm': sb = 0x10D4; break;
			case 'n': sb = 0x1050; break;
			case 'o': sb = 0x00DC; break;
			case 'p': sb = 0x0170; break;
			case 'q': sb = 0x0486; break;
			case 'r': sb = 0x0050; break;
			case 's': sb = 0x0888; break;
			case 't': sb = 0x0078; break;
			case 'u': sb = 0x001C; break;
			case 'v': sb = 0x0804; break;
			case 'w': sb = 0x2814; break;
			case 'x': sb = 0x28C0; break;
			case 'y': sb = 0x080C; break;
			case 'z': sb = 0x2048; break;
			case '!': sb = 0x0006; break;
			case '\"': sb = 0x0220; break;
			case '#': sb = 0x12CE; break;
			case '$': sb = 0x12ED; break;
			case '%': sb = 0x2424; break;
			case '&': sb = 0x0B5D; break;
			case '\'': sb = 0x0400; break;
			case '(': sb = 0x0C00; break;
			case ')': sb = 0x2100; break;
			case '*': sb = 0x3FC0; break;
			case '+': sb = 0x12C0; break;
			case ',': sb = 0x2000; break;
			case '-': sb = 0x00C0; break;
			case '.': sb = 0x0000; break;
			case '/': sb = 0x2400; break;
			case ':': sb = 0x1200; break;
			case ';': sb = 0x2200; break;
			case '<': sb = 0x0C00; break;
			case '=': sb = 0x00C8; break;
			case '>': sb = 0x2100; break;
			case '?': sb = 0x1083; break;
			case '@': sb = 0x02BB; break;
			case '[': sb = 0x0039; break;
			case '\\': sb = 0x0900; break;
			case ']': sb = 0x000F; break;
			case '^': sb = 0x2403; break;
			case '_': sb = 0x0008; break;
			case '`': sb = 0x0100; break;
			case '{': sb = 0x2149; break;
			case '|': sb = 0x1200; break;
			case '}': sb = 0x0C89; break;
			case '~': sb = 0x0520; break;
			default: sb = 0x0000; break;
			}
			return sb;
		}

		public SegmentPosition GetSegmentFlag(ushort seg)
		{
			ushort res = 0;
			for(int i = 0; i < Segments; i++) {
				if ((seg >> i & 0x1) != 0) res |= (ushort)(0x1 << i);
			}
			return (SegmentPosition)res;
		}

		public void SetAscii(int pos, char ascii)
		{
			SetSegmentAsBits(pos, GetAsciiSegment(ascii));
		}

		public void SetAsciiString(string str)
		{
			int sl = str.Length;
			if (sl > 8) sl = 8;
			for (int i = 0; i < 8; i++) {
				if (i < sl) {
					SetAscii(i, str[i]);
				} else {
					SetSegment(i, SegmentPosition.None);
				}
			}
		}

		public void ShiftSegmentsLeft()
		{
			for(int i = 0; i < Digits - 1; i++) {
				for(int j = 0; j < Segments; j++) {
					dins[(int)segarray[i, j]] = dins[(int)segarray[i + 1, j]];
				}
			}
			for (int j = 0; j < Segments; j++) dins[(int)segarray[Digits - 1, j]] = GpioPinValue.Low;
		}

		public void ShiftSegmentsRight()
		{
			for (int i = Digits - 1; i >= 1; i--) {
				for (int j = 0; j < Segments; j++) {
					dins[(int)segarray[i, j]] = dins[(int)segarray[i - 1, j]];
				}
			}
			for (int j = 0; j < Segments; j++) dins[(int)segarray[0, j]] = GpioPinValue.Low;
		}

		public void ShiftSegments(bool left)
		{
			if (left == true) {
				ShiftSegmentsLeft();
			} else {
				ShiftSegmentsRight();
			}
		}

		public void SetMark(AM50288H_Mark mark, bool set)
		{
			dins[(int)mark] = (set == true) ? GpioPinValue.High : GpioPinValue.Low;
		}
	}
}
